function [MLthetahat,thetahats]=anovadip(nucfreqs,loci,pops,a,b)

%Code copyright 2002 by M. B. Hamilton, J. R. Miller and Georgetown University.
%Code may be modified as long as credit is given to the original authors.

% anova estimate Fst for for diploids
% follows Weir pp. 176-179 
% inputs:
% nucfreqs: contains proportions of genotypes (output of pickindivs or picksubpops)
% loci: vector specifying which loci
% pops: vector specifying subpopulations
% a=number of populations
% b=sample size (same for each subpopulation; thus we ASSUME BALANCED DATA)
% outputs:
% thetahat is point estimate of Fst, combining estimates for each locus
% s1 contains numerators in estimators of Fst at each locus (Weir p. 178), needed for bootstrap
% s2 contains denominators
% thetahats contains vector of Fst values (one for each locus specified)

numloci = length(loci);
n=b*ones(size(pops)); % sample sizes, assumes all loci have same number of individuals
ntot=sum(n); % total sample size of individuals
r=length(pops); %number of populations sampled

dfi=ntot-r; % degrees of freedom  between individuals within subpopulations

PAA=nucfreqs(pops,loci,1); % proportions of AA genotypes within subpopulations
p=PAA + 0.5*nucfreqs(pops,loci,2); %proportions of allele A within subpopulations, PAA + 1/2PAa

dfp=r-1; %degrees of freedom between populations
pmean=(n*p)/ntot; %mean proportion of allele A for each locus

%check to see if pmean = 0 or 1, if so theta is zero, else estimate theta
for i=1:numloci
    locus = loci(1,i);  %look at the locus specified by the vector of loci
	if (pmean(1,locus) == 0) | (pmean(1,locus) == 1)    %sample of pops all have zero or one freq of A
        %both s1 and s2 are zero since there is no genetic variation
        s1(1,i)=0.0; %numerators in estimators of Fst at each locus (Weir p. 178)
		s2(1,i)=0.0; %denominators

        thetahats(1,i)=0.0; %assign value directly to avoid div by zero
        
    else    %no error condition, so estimate
        msi=(n*(p(:,locus)+PAA(:,locus)-(2*p(:,locus).^2)))/dfi; %Weir p. 177
        
        msp=2*(n*(p(:,locus)-ones(r,1)*pmean(1,locus)).^2)/dfp; %Weir p. 177, was missing factor of 2!!
		dfg=ntot; %degrees of freedom within individuals
		msg=(n*(p(:,locus)-PAA(:,locus)))/dfg; %Weir p. 177
		
		nc=(ntot-sum(n.^2)/ntot)/dfp; %weight for unequal pop. sizes (p. 172)
        
		s1(1,i)=msp-msi; %numerators in estimators of Fst at each locus (Weir p. 178)
		s2(1,i)=msp+(nc-1).*msi+nc.*msg; %denominators
		
		thetahats(1,i)=s1(1,i)/s2(1,i); %estimators of Fst *for each locus separately* (p. 178)
    end 
end

MLs1=sum(s1);   %multilocus numerator
MLs2=sum(s2);   %multilocus denominator
if MLs2==0
    MLthetahat = 0.0; %assign value directly to avoid div by zero, no genetic variation at all loci
else
    MLthetahat=MLs1/MLs2; %estimator of Fst, combining loci (p. 179)
end

thetahats = thetahats'; %transpose to make output match with rountines in Main