function [pnuc, porg, snucgen, sorggen] = evolvepops(pnuc, porg, nnloci, norgloci, npops, alpha, N, t, mp, ms, gens);

%Code copyright 2002 by M. B. Hamilton, J. R. Miller and Georgetown University.
%Code may be modified as long as credit is given to the original authors.

%function to evolve set of populations under drift & migration, with possible selfing
%N is effective size of a single subpopulation, all subpops are same size
%t is the outcrossing rate, npops is the number of subpopulations
%mp is the pollen migration rate, ms is the seed migration rate in proportion of subpop that migrates
%assumes nuclear loci have two alleles with initial frequency initpnuc where initpnuc is a vector with 1 to npops entries
%gens is the number of generations
%drift occurs first, then pollen migration, then mating, then seed migration
%nnloci is the number of nuclear loci, norgloci the number of organelle loci
%version 1.2 03/27/01 

%vars to test, all this can be cut once function is called externally
% nnloci = 2; %number of nuclear loci
% norgloci = 2;  %number of organelle loci
% npops=8;    %number of populations
% initpnuc = [0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5];   %these init vectors have as many values as there are pops
% initporg = [0.5 0.5 0.5 0.5 0.5 0.5 0.5 0.5];   %initial organelle frequencies
% alpha = [1.0; 1.0];   %maternal transmission rate, norgloci by 1 column vector, one value for each organelle locus
% N=20;       %number of individuals within each population
% t=1.0;      %outcrossing rate
% mp=0.0;     %pollen migration rate
% ms=0.1;     %seed migration rate
% gens=100;     %number of generations to evolve
% 
% pnuc = [];
% for i=1:nnloci
%     pnuc = [pnuc; initpnuc];   %make nnloci by npops matrix of initial values
% end %for nloci loop
% 
% porg = [];
% for i=1:norgloci
%     porg = [porg; initporg];     %make norgloci by npops matrix of initial values
% end %for norgloci loop
%end test vars, stop cut here

S = rand('state');
rand('state',sum(100*clock));   %use clock value for seed in random number generator
for i=1:gens
    
    %data to plot one allele frequency as they drift
%     nucvec(i) = pnuc(1,1); %nuc p for one pop
%     orgvec(i) = porg(1,1); %org p for one pop
%     xvec(i) = i;   %make vector of generation we are on for x axis
        
    %let populations drift
    %'initial'
    %porg
    [pnuc] = drift(pnuc, nnloci, npops, 2*N);
    %'after drift'
    [porg] = drift(porg, norgloci, npops, N);
    %call function [driftmatrix] = drift(pmatrix, nloci, npops, numalleles);
    
    %get average allele frequencies
    [meanpnuc] = avgfreq(pnuc, nnloci, npops);
    [meanporg] = avgfreq(porg, norgloci, npops);
    %call function [meanfreq] = avgfreq(pmatrix, nloci, npops);
    
    %get allele frequencies after pollen dispersal
    %[pollennucfreq, pollenorgfreq] = dispersepollen(pnuc, porg, alpha, mp, meanpnuc, meanporg, nnloci, norgloci, npops, t);
    %call function [nucfreq, orgfreq] = dispersepollen(pnuc, porg, alpha, mp, meanpnuc, meanporg, nnloci, norgloci, npops, t);
    [pollennucfreq, pollenorgfreq] = dispersepollen2(pnuc, porg, alpha, mp, meanpnuc, meanporg, nnloci, norgloci, npops, t, N);
    %call function [nucfreq, orgfreq] = dispersepollen2(pnuc, porg, alpha, mp, meanpnuc, meanporg, nnloci, norgloci, npops, t, N);

    %make genotypes from allele frequencies
    [nucgen, orggen] = getgenotypes(pollennucfreq, pollenorgfreq, nnloci, norgloci, npops, N, t);
    %call function [nucgen, orggen] = getgenotypes(pnuc, porg, nnloci, norgloci, npops, N, t);    

    %get genotype counts after seed dispersal
    [snucgen, sorggen] = disperseseed(nucgen, orggen, nnloci, norgloci, npops, N, t, ms);    
    %call function [snucgen, sorggen] = disperseseed(nucgen, orggen, nnloci, norgloci, npops, N, t, ms);    
    %'after seed dispersal'
    %sorggen
    
    %make genotype counts into allele frequency so we can repeat process
    [pnuc, porg] = getallelefreq(snucgen, sorggen, nnloci, norgloci, npops, N);
    %call function [nucfreq, orgfreq] = getallelefreq(nucgen, orggen, nnloci, norgloci, npops, N);
    %'after converting from genotype counts'
    %porg

end %end generations loop
% plot(xvec,nucvec,'r+:');
% axis([1 gens 0.0 1.0]);
% hold on;
% plot(xvec,orgvec,'kx:');
% axis([1 gens 0.0 1.0]);
% hold off;
%___________________________
function [driftmatrix] = drift(pmatrix, nloci, npops, numalleles);
%populations drift, uses Kimura's psuedo random variable method.
%pmatirx is the original matrix of p values for each population (columns)
%and for each locus (rows).  nloci is the number of loci, npops is the number
%of populations, numalleles is the number of alleles sampled (2*N for nuclear alleles
%and N for organelle alleles).
    driftmatrix = pmatrix;
    for k=1:nloci
        for j=1:npops %allele frequencies drift
            
            delta = (2*rand - 1)*sqrt((3*pmatrix(k,j)*(1-pmatrix(k,j)))/(numalleles));
            if (pmatrix(k,j) + delta)> 1.0
                driftmatrix(k,j) = 1.0;
            elseif (pmatrix(k,j) + delta)<0.0
                driftmatrix(k,j) = 0.0;
            else
                driftmatrix(k,j) = pmatrix(k,j) + delta;
            end
            
        end %end npops loop
    end %end loci loop
%___________________________
function [meanfreq] = avgfreq(pmatrix, nloci, npops);
%calculate average allele frequencies for each locus among populations.
%pmatirx is the original matrix of p values for each population (columns)
%and for each locus (rows).  nloci is the number of loci, npops is the number
%of populations.
%meanfreq is a nloci by 1 column vector with mean p over all pops at each locus.

    for k=1:nloci
        sump(k) = 0.0;
        %sumporg(k) = 0.0
        for j=1:npops %get sum allele frequency over pops
            sump(k) = sump(k) + pmatrix(k,j);
        end %end npops loop
        meanfreq(k,1) = sump(k)/npops;  %get average allele frequency
    end %end loci loop
%___________________________
function [nucfreq, orgfreq] = dispersepollen(pnuc, porg, alpha, mp, meanpnuc, meanporg, nnloci, norgloci, npops, t);
%pollen dispersal, assumes random migration from average pollen pool.
%allele frequencies change by fraction of migration times mean p among pops
%1-mp is fraction not migrated, mp is fraction due to pollen migration

%need to change migration model since migration is integral, not fractional
%entire alleles migrate, not fractions of alleles
%need to base migration on chance of A or a allele migrating based on mean allele frequency
%also need to add feature to allow migration at random intervals if mp*N is less than one allele

    pprime = 0.0;
    nucfreq = zeros(nnloci,npops);
    for k=1:nnloci
        for j=1:npops
            migfactor = t*mp;
            %proportion of nuclear alleles that are transmitted in pollen
            %from other pops times the outcrossing rate 
            pprime = ((1-migfactor)*pnuc(k,j)) + (migfactor*meanpnuc(k,1));
            if pprime > 1.0
                nucfreq(k,j) = 1.0;
            elseif pprime < 0.0
                nucfreq(k,j) = 0.0;
            else
                nucfreq(k,j) = pprime;
            end
        end %end npops loop
    end %end loci loop
    
    pprime = 0.0;
    orgfreq = zeros(norgloci,npops);
    for k=1:norgloci
        for j=1:npops
            %porg(k,j)   %show to test
            %meanporg(k,1)   %show to test
            migfactor = (1-alpha(k,1))*t*mp;
            %proportion of organelles that are transmitted paternally times
            %the outcrossing rate times the amount of pollen migration from other pops 
            pprime = ((1-migfactor)*porg(k,j)) + (migfactor*meanporg(k,1));
            if pprime > 1.0
                orgfreq(k,j) = 1.0;
            elseif pprime < 0.0
                orgfreq(k,j) = 0.0;
            else
                orgfreq(k,j) = pprime;
            end
        end %end npops loop
    end %end loci loop
%___________________________
function [nucgen, orggen] = getgenotypes(pnuc, porg, nnloci, norgloci, npops, N, t);    
%make genotypes from allele frequencies
%nuclear genotype matrix has three rows for each genotytpe (AA, Aa, aa) per locus
%organelle genotype matrix has one row for each genotytpe (A) per locus
%each column is one population in both cases
%mating is random for t percent of genotypes and selfing for 1-t percent
    
    %make nuclear genotypes
    nucgen = zeros(3*nnloci,npops);
    i = 0;  %index for rows in pnuc
    for k=1:3:3*nnloci
        i = i+1;
        for j=1:npops
            het = round(t*(2*pnuc(i,j)*(1-pnuc(i,j)))*N);    %number of Aa that mate randomly
            selfedhet = round((1-t)*(2*pnuc(i,j)*(1-pnuc(i,j)))*N);  %number of Aa that self
            nucgen(k,j) = round(pnuc(i,j)^2*N)+round(0.25*selfedhet);    %number of AA
            nucgen(k+1,j) = het+round(0.50*selfedhet);    %number of Aa
            nucgen(k+2,j) = round((1-pnuc(i,j))^2*N)+round(0.25*selfedhet);    %number of aa
        end %end npops loop
    end %end loci loop
    
    %make organelle genotypes
    orggen = zeros(norgloci,npops);
    for k=1:norgloci
        for j=1:npops
            orggen(k,j) = round(porg(k,j)*N);    %number of A haplotypes
        end %end npops loop
    end %end loci loop

    %now check total number of nuclear genotypes for rounding error
    %all genotypes at one locus should sum to N
    adjustment = zeros(nnloci,npops);
    i=0;
    for k=1:3:3*nnloci
        i=i+1;
        for j=1:npops
            sum = nucgen(k,j)+nucgen(k+1,j)+nucgen(k+2,j); %sum AA, Aa and aa
            factor = sum/N;
            if factor == 1.0 %N genotypes, no adjustment needed
                adjustment(i,j) = 0; 
            elseif factor > 1.0 %more than N genotypes, need to subtract
                adjustment(i,j) = -rem(sum,N);
            elseif factor < 1.0 %fewer than N genotypes, need to add
                adjustment(i,j) = rem(N,sum);
            else    %just in case
                adjustment = 0;
            end %if statements
        end %end npops loop
    end %end loci loop

    %now actually adjust number of nuclear genotypes (randomly selected) if necessary
    for k=1:nnloci
        row = (k*3) - 2;    %assumes 3 nuc genotypes
        for j=1:npops
            if adjustment(k,j) ~= 0  %do only if adjustment needed
                for i=1:abs(adjustment(k,j))
                    random = ceil(1000*rand);   %get a random number and round it up
                    pick = row + rem(random,3);    %randomly select one genotype to change count, gives 0, 1 or 2 as result
                    if (adjustment(k,j) > 0)&(nucgen(pick,j)<N)  %need to add a genotype but only if num genotypes less than N
                        nucgen(pick,j) = nucgen(pick,j) + 1;
                    elseif (adjustment(k,j) < 0)&(nucgen(pick,j)>0)     %need to subtract a genotype but only if num genotypes greater than zero
                        nucgen(pick,j) = nucgen(pick,j) - 1;
                    else
                        i = i - 1;  %did not make change so try again
                    end %if statement
                end %end adjustment loop
            end %if statement
        end %end npops loop
    end %end loci loop
%___________________________
function [snucgen, sorggen] = disperseseed(nucgen, orggen, nnloci, norgloci, npops, N, t, ms);    
%disperse genotypes via seed.
%migrant pool is average frequency of each genotype among all populations.
%nucgen and orggen are matrices of genotype counts.
    snucgen = nucgen; %allocate space for post-seed dispersal genotype counts
    sorggen = [2*norgloci,npops];   %allocate space for seed dispersal haplotype counts for both A and a
    k=0;
    for i=1:2:2*norgloci    %fill in sorggen values for both A and a haplotypes
        k = k+1;
        for j=1:npops
            sorggen(i,j) = orggen(k,j);
            sorggen(i+1,j) = N - orggen(k,j);
        end
    end
    
    %get average frequency of each nuclear genotype for each locus
    avgnucgen = zeros(3*nnloci,1);   %a column vector of average genotype frequencies
    for k=1:3:3*nnloci
        sumAA = 0;
        sumAa = 0;
        sumaa = 0;
        for j=1:npops
            sumAA = sumAA + nucgen(k,j);
            sumAa = sumAa + nucgen(k+1,j);
            sumaa = sumaa + nucgen(k+2,j);
        end %end npops loop
        %k
        avgnucgen(k,1) = round(sumAA/npops);
        avgnucgen(k+1,1) = round(sumAa/npops);
        avgnucgen(k+2,1) = round(sumaa/npops);
    end %end loci loop

    %get average frequency of A and a haplotypes for each locus
    avgorggen = zeros(2*norgloci,1);   %a column vector of average genotype frequencies
    i = 1;
    for k=1:norgloci
        sumA = 0;
        suma = 0;
        for j=1:npops
            sumA = sumA + orggen(k,j);
            suma = suma + (N-orggen(k,j));
        end %end npops loop
        avgorggen(i,1) = round(sumA/npops);
        avgorggen(i+1,1) = round(suma/npops);
        i=i+2;
    end %end loci loop
    
    %determine integral numbers of migration events and chance of fractional migration
    individmig = fix(N*ms); %round toward zero to get integral migration events
    fractmig = mod(N*ms,1); %get fractional migration events

    %allocate space for index range matrix of average nuclear genotypes
    range = [1,3];
    outrange = [1,3];

    %exchange nuclear genotypes from each pop with total population average
    for k=1:3:3*nnloci
        range(1,1) = avgnucgen(k,1);   %upper bound for average AA
        range(1,2) = avgnucgen(k,1) + avgnucgen(k+1,1);  %upper bound for average Aa
        range(1,3) = N;  %upper bound for average aa
            
        for j=1:npops
            %select genotypes that are outgoing migrants
            outrange(1,1) = nucgen(k,j);   %upper bound for AA in pop j
            outrange(1,2) = nucgen(k,j) + nucgen(k+1,j);  %upper bound for Aa in pop j
            outrange(1,3) = N;  %upper bound for aa in pop j
                
            %allow migration at random intervals if ms*N is less than one individual
            individmig = fix(N*ms); %round toward zero to get integral migration events
            if fractmig > 0     %if fractional migration is greater than zero
                chance = rand;  %determine chances of migration event for each population
                if chance <= fractmig
                    individmig = individmig + 1;    %add one migration event
                end
            end
                
            for i=1:individmig  %select outgoing migrants to subtract
                genotype = ceil(N*rand);   %get random integer between 1 and N
                if (snucgen(k,j)>0)&(genotype<=outrange(1,1))   %genotype is in range of AA for pop j
                    snucgen(k,j) = snucgen(k,j) - 1;    %subtract one AA genotype
                elseif (snucgen(k+1,j)>0)&(genotype>outrange(1,1))&(genotype<=outrange(1,2))
                    snucgen(k+1,j) = snucgen(k+1,j) - 1;    %subtract one Aa genotype
                elseif (snucgen(k+2,j)>0)&(genotype>outrange(1,2))&(genotype<=outrange(1,3))
                    snucgen(k+2,j) = snucgen(k+2,j) - 1;    %subtract one aa genotype
                else
                    i = i-1;    %did not change anything so try again
                end
            end %end individmig loop
             
            for i=1:individmig  %select incoming migrants to add
                genotype = ceil(N*rand);   %get random integer between 1 and N
                if (snucgen(k,j)<N)&(genotype<=range(1,1))   %genotype is in range of population average AA, but fewer than N AA genotypes
                    snucgen(k,j) = snucgen(k,j) + 1;    %add one AA genotype
                elseif (snucgen(k+1,j)<N)&(genotype>range(1,1))&(genotype<=range(1,2)) %genotype is in range of population average Aa, but fewer than N Aa genotypes
                    snucgen(k+1,j) = snucgen(k+1,j) + 1;    %add one Aa genotype
                elseif (snucgen(k+2,j)<N)&(genotype>range(1,2))&(genotype<=range(1,3)) %genotype is in range of population average aa, but fewer than N aa genotypes
                    snucgen(k+2,j) = snucgen(k+2,j) + 1;    %add one aa genotype
                else
                    i = i-1;    %did not change anything so try again    
                end
            end %end individmig loop
        end %end npops loop
    end %end loci loop
    
    %exchange organelle genotypes from each pop with total population average
    for k=1:2:2*norgloci
        for j=1:npops
        %select haplotypes that are outgoing migrants
        
            %allow migration at random intervals if ms*N is less than one individual
            individmig = fix(N*ms); %round toward zero to get integral migration events
            if fractmig > 0     %if fractional migration is greater than zero
                chance = rand;  %determine chances of migration event for each population
                if chance <= fractmig
                    individmig = individmig + 1;    %add one migration event
                end
            end

            for i=1:individmig  %select outgoing migrants to subtract
                genotype = ceil(N*rand);   %get random integer between 1 and N
                if (sorggen(k,j)>0)&(genotype<=sorggen(k,j))   %genotype is in range of A for pop j, but there are more than zero haplotypes
                    sorggen(k,j) = sorggen(k,j) - 1;    %subtract one A haplotype
                elseif (sorggen(k+1,j)>0)&(genotype>sorggen(k,j))&(genotype<=N)
                    sorggen(k+1,j) = sorggen(k+1,j) - 1;    %subtract one a haplotype
                else
                    i = i-1;    %did not change anything so try again
                end
            end %end individmig loop
                
            for i=1:individmig  %select migrants to add
                genotype = ceil(N*rand);   %get random integer between 1 and N
                if (sorggen(k,j)<N)&(genotype<=avgorggen(k,1))   %genotype is in range of population average A
                    sorggen(k,j) = sorggen(k,j) + 1;    %add one A haplotype
                elseif (sorggen(k+1,j)<N)&(genotype>avgorggen(k,1))   %genotype is in range of population average a
                    sorggen(k+1,j) = sorggen(k+1,j) + 1;    %add one a haplotype
                else
                    i = i-1;    %did not change anything so try again
                end
            end %end individmig loop

        end %end npops loop
    end %end loci loop
%___________________________
function [nucfreq, orgfreq] = getallelefreq(nucgen, orggen, nnloci, norgloci, npops, N);    
%input genotype count matrix and compute frequency of A for each population
%uses allele counting method for nuclear alleles since populations may not be in HWE
    
    orgfreq = zeros(norgloci,npops);    %allocate space for result
    nucfreq = zeros(nnloci,npops);
    
    k = 0;
    for i=1:3:3*nnloci
        k = k + 1;
        for j=1:npops
            sum = 2*nucgen(i,j) + nucgen(i+1,j);    %add 2 times AA plus Aa to count A alleles
            nucfreq(k,j) = sum/(2*N);
        end %end npops loop
    end %end loci loop

    k = 0;
    for i=1:2:2*norgloci
        k = k+1;    %counter
        for j=1:npops
            orgfreq(k,j) = orggen(i,j)/N;  %get frequency of A as count of A haplotype div N
        end %end npops loop
    end %end loci loop
%___________________________
function [nucfreq, orgfreq] = dispersepollen2(pnuc, porg, alpha, mp, meanpnuc, meanporg, nnloci, norgloci, npops, t, N);
%disperse alleles via pollen, assumes random migration from average pollen pool.
%allele counts change by migration from average allele count among all pops at one locus
%1-mp is fraction not migrated, mp is fraction due to pollen migration
%migrant pool is average number of each allele among all populations.

    %allocate space for allele counts
    nucallele = [2*nnloci,npops]; 
    orgallele = [2*norgloci,npops];

    %count nuclear alleles in each population
    j = 0;
    for i=1:2:2*nnloci
        j = j + 1;
        for k=1:npops
            nucallele(i,k) = round(N*pnuc(j,k));    %number of A pollen alleles in pop j
            nucallele(i+1,k) = N - nucallele(i,k);  %number of a pollen alleles in pop j
        end %for npops
    end %for nnloci
    
    %count organelle alleles in each population
    j = 0;
    for i=1:2:2*norgloci
        j = j + 1;
        for k=1:npops
            orgallele(i,k) = round(N*porg(j,k));    %number of A pollen haplotypes in pop j
            orgallele(i+1,k) = N - orgallele(i,k);  %number of a pollen haplotypes in pop j
        end %for npops
    end %for nnloci
    
    %get average numbers of alleles A and a for each locus
    avgnucallele = zeros(2*nnloci,1);   %a column vector of average numbers of nuc alleles, A then a for each locus
    j = 0;
    for k=1:2:2*nnloci
        j = j + 1;
        avgnucallele(k,1) = round(N*meanpnuc(j,1));      %avg number of A alleles
        avgnucallele(k+1,1) = N-avgnucallele(k,1);    %avg number of a alleles
    end %end loci loop

    %get average numbers of A and a haplotypes for each locus
    avgorgallele = zeros(2*norgloci,1);   %a column vector of average numbers of org alleles, A then a for each locus
    j = 0;
    for k=1:2:2*norgloci
        j = j + 1;
        avgorgallele(k,1) = round(N*meanporg(j,1));      %avg number of A haplotypes
        avgorgallele(k+1,1) = N-avgorgallele(k,1);    %avg number of a haplotypes
    end %end loci loop
    
    %determine integral numbers of migration events and chance of fractional migration for nuclear loci
    rate = N*mp*t;
    individmig = fix(rate); %round toward zero to get integral migration events
    fractmig = mod(rate,1); %get fractional migration events

    %exchange nuclear genotypes from each pop with total population average
    for k=1:2:2*nnloci
        for j=1:npops
            %allow migration at random intervals if mp*N is less than one individual
            individmig = fix(rate); %round toward zero to get integral migration events
            if fractmig > 0     %if fractional migration is greater than zero
                chance = rand;  %determine chances of migration event for each population
                if chance <= fractmig
                    individmig = individmig + 1;    %add one migration event
                end
            end

            for i=1:individmig  %select outgoing migrants to subtract
                allele = ceil(N*rand);   %get random integer between 1 and N
                if (nucallele(k,j)>0) & (allele<=nucallele(k,j))   %allele is in range of A for pop j
                    nucallele(k,j) = nucallele(k,j) - 1;    %subtract one A allele
                elseif (nucallele(k+1,j)>0)&(allele>nucallele(k,j))&(allele<=N)   %allele is in range of a for pop j
                    nucallele(k+1,j) = nucallele(k+1,j) - 1;    %subtract one a allele
                else
                    i = i-1;    %did not change anything so try again
                end
            end %end individmig loop

            for i=1:individmig  %select incoming migrants to add
                allele = ceil(N*rand);   %get random integer between 1 and N
                if (nucallele(k,j)<N)&(allele<=avgnucallele(k,1))   %allele is in range of population average A
                    nucallele(k,j) = nucallele(k,j) + 1;    %add one A allele
                elseif (nucallele(k+1,j)<N)&(allele>avgnucallele(k,1))&(allele <= N) %allele is in range of population average a
                    nucallele(k+1,j) = nucallele(k+1,j) + 1;    %add one a allele
                else
                    i = i-1;    %did not change anything so try again
                end
            end %end individmig loop

        end %end npops loop
    end %end loci loop
    
    %exchange organelle genotypes from each pop with total population average
    x = 0;
    for k=1:2:2*norgloci
        x = x + 1;
        %determine integral numbers of migration events and chance of fractional migration for organelle loci
        %chances of migration are locus specific since transmission rate can vary among organelle loci
        rate = N*mp*t*(1-alpha(x,1));
        individmig = fix(rate); %round toward zero to get integral migration events
        fractmig = mod(rate,1); %get fractional migration events
        
        for j=1:npops
            %allow migration at random intervals if mp*N is less than one individual
            individmig = fix(rate); %round toward zero to get integral migration events
            if fractmig > 0     %if fractional migration is greater than zero
                chance = rand;  %determine chances of migration event for each population
                if chance <= fractmig
                    individmig = individmig + 1;    %add one migration event
                end
            end

            for i=1:individmig  %select outgoing migrants to subtract
                allele = ceil(N*rand);   %get random integer between 1 and N
                if (orgallele(k,j)>0) & (allele<=orgallele(k,j))   %allele is in range of A for pop j
                    orgallele(k,j) = orgallele(k,j) - 1;    %subtract one A allele
                elseif (orgallele(k+1,j)>0)&(allele>orgallele(k,j))&(allele <= N)   %allele is in range of a for pop j
                    orgallele(k+1,j) = orgallele(k+1,j) - 1;    %subtract one a allele
                else
                    i = i-1;    %did not change anything so try again
                end
            end %end individmig loop

            for i=1:individmig  %select incoming migrants to add
                allele = ceil(N*rand);   %get random integer between 1 and N
                if (orgallele(k,j)<N)&(allele<=avgnucallele(k,1))   %allele is in range of population average A
                    orgallele(k,j) = orgallele(k,j) + 1;    %add one A allele
                elseif (orgallele(k+1,j)<N)&(allele>avgnucallele(k,1))&(allele<=N) %allele is in range of population average a
                    orgallele(k+1,j) = orgallele(k+1,j) + 1;    %add one a allele
                else
                    i = i-1;    %did not change anything so try again
                end
            end %end individmig loop

        end %end npops loop
    end %end loci loop
    
    %now make post-pollen dispersal allele counts into allele and haplotype frequencies
    %one half of the nuclear alleles do not change since they are in ovules and do not disperse
    k = 0;
    for i=1:2:2*nnloci
        k = k + 1;
        for j=1:npops
            nucfreq(k,j) = 0.5*(pnuc(k,j)) + 0.5*(nucallele(i,j)/N);    %frequency of A allele in pop j
        end %for npops
    end %for nnloci

    k = 0;
    for i=1:2:2*norgloci
        k = k + 1;
        for j=1:npops
            orgfreq(k,j) = orgallele(i,j)/N;    %frequency of A haplotype in pop j
        end %for npops
    end %for nnloci
%___________________________
